# -*- coding: UTF-8 -*-

from artix.common import py2to3utils
from artix.common.py2to3utils import configparser
from singleton import Singleton
from infoReader import infoReader
import logger
import os
import re


loggerSync = logger.getLogger()


class PropertyReader(object):
    __metaclass__ = Singleton
    configPath = os.path.normpath('%s/../properties/cash.ini' % os.path.dirname(__file__))

    def __init__(self):
        self.config = configparser.RawConfigParser()
        self.remoteIp = None
        self.unloadScript = '/linuxcash/cash/exchangesystems/exchangers/src/unload/service/cash/startReader.py'
        self.cashCode = None
        self.storeID = None
        self.localBroker = 'guest/guest@127.0.0.1:5672'
        self.remotePort = 5672
        self.maxCountBackupFiles = 0
        self.actualCountDaysForBackupFiles = 0
        self.cashServerRestPort = 8080
        self.mode = 'cash'
        self.uploadScript = '/linuxcash/cash/exchangesystems/exchangers/src/upload/converter/aif.py'
        self.useQueueInUpload = True
        self.brokerConnectionHeartbeat = 0
        self.dictUrlHostCS = '127.0.0.1'
        self.dictUrlPortCS = 80
        self.clusterId = ''
        self.grpcHostCS = '127.0.0.1'
        self.grpcPortCS = 10001
        self.grpcCaCertPath = None
        self.protocolVersion = 3
        self.unloadedSalesDirName = 'unloadedSales'
        self.unloadSalesRequestThroughCS = False
        self.isSendDebugDictStatus = True
        self.isSendDebugSaleStatus = True
        self.httpHostCS = self.remoteIp
        self.httpPortCS = self.cashServerRestPort
        self.fileQueueRoot = '/linuxcash/cash/data/queues' if self.mode == 'cash' else '/opt/queues'
        self.queueChunkSize = 25000
        self.remoteBroker = 'guest/guest@%s:%s' % (self.remoteIp, self.remotePort)
        self.cashServerId = ''

    def init(self):
        loggerSync.info(u'Инициализация настроек sync-agent')

        # Считывание настроек подключения sync-агента
        if not os.access(PropertyReader.configPath, os.F_OK):
            loggerSync.error(u'Нет файла с настройками')
            return False

        self.config.read(PropertyReader.configPath)
        try:
            # Значения, которые обязательно должны присутствовать в конф-ом файле
            self.remoteIp = self.__getValue('dept.broker.ip', None, True)
            self.unloadScript = self.__getValue('unloadScript', None, True)
            self.cashCode = self.__getValue('cash.code', None, True)
            self.storeID = self.__getValue('storeID', None, True)
        except Exception as e:
            loggerSync.error(str(e))
            return False

        self.localBroker = self.__getValue('localBroker', 'guest/guest@127.0.0.1:5672')
        self.remotePort = self.__getValue('dept.broker.port', 5672)
        self.maxCountBackupFiles = self.__getValue('maxCountBackupFiles', 0)
        self.actualCountDaysForBackupFiles = self.__getValue('actualCountDaysForBackupFiles', 0)
        self.cashServerRestPort = self.__getValue('cashServerRestPort', 8080)
        self.mode = self.__getValue('mode', 'cash')
        self.uploadScript = self.__getValue('uploadScript', '/linuxcash/cash/exchangesystems/exchangers/src/upload/converter/aif.py')
        self.useQueueInUpload = self.__getValue('useQueueInUpload', True)
        self.brokerConnectionHeartbeat = self.__getValue('brokerConnectionHeartbeat', 0)
        self.dictUrlHostCS = self.__getValue('dictUrlHostCS', '127.0.0.1')
        self.dictUrlPortCS = self.__getValue('dictUrlPortCS', 80)
        self.clusterId = self.__getValue('clusterId', '')
        self.grpcHostCS = self.__getValue('grpcHostCS', '127.0.0.1')
        self.grpcPortCS = self.__getValue('grpcPortCS', 10001)
        self.grpcCaCertPath = self.__getValue('grpcCaCertPath', None)
        self.protocolVersion = self.__getValue('protocolVersion', 3)
        # в серверном режиме на ubunu 18.04 версия протокола не может быть меньше 5, т.к. на сервере отказались от qpid
        if self.mode == 'server' and (infoReader.platformIsBionic or infoReader.package == 'artixcs-sync') and self.protocolVersion < 5:
            self.protocolVersion = 5
        self.unloadedSalesDirName = self.__getValue('unloadedSalesDirName', 'unloadedSales')
        self.unloadSalesRequestThroughCS = self.__getValue('unloadSalesRequestThroughCS', False)
        self.isSendDebugDictStatus = self.__getValue('isSendDebugDictStatus', True)
        self.isSendDebugSaleStatus = self.__getValue('isSendDebugSaleStatus', True)
        self.httpHostCS = self.__getValue('httpHostCS', self.remoteIp)  # почему-то параметр с КС передается, но на sync по обычаю игнорировался
        self.httpPortCS = self.__getValue('httpPortCS', self.cashServerRestPort)  # просто переименовал параметр, чтоб он совпадал со значением из SyncConfigInfo
        self.fileQueueRoot = self.__getValue('fileQueueRoot', '/linuxcash/cash/data/queues' if self.mode == 'cash' else '/opt/queues')
        self.queueChunkSize = self.__getValue('queueChunkSize', 25000)
        self.remoteBroker = 'guest/guest@%s:%s' % (self.remoteIp, self.remotePort)
        return True

    def update(self, configInfo):
        if not configInfo:
            return False

        for k, v in configInfo.items():
            configInfo[k] = v if not isinstance(v, py2to3utils.string_types) else py2to3utils.ensureStr(v)

        if self.cashServerId and configInfo.get("cashServerId") and \
                (configInfo.get("cashServerId") != self.cashServerId and not configInfo.get("isRewriteConfig", False)):
            loggerSync.info("The server %s tried to overwrite the settings, "
                            "but it is not the owner server. Owner server is now %s"
                            % (configInfo.get("cashServerId"), self.cashServerId))
            return False

        def simpleProperty(configInfoValue, syncPropertyName, syncAttrName):
            syncPropertyValue = self.__getattribute__(syncAttrName)
            if configInfoValue is not None and syncPropertyValue != configInfoValue:
                loggerSync.info(u'Изменяем настройку %s: %s -> %s' % (syncPropertyName, syncPropertyValue, configInfoValue))
                self.__setattr__(syncAttrName, configInfoValue)
                self.__rewriteConfig(syncPropertyName, configInfoValue)
                return True
            return False

        changed = False
        changed |= simpleProperty(configInfo.get('cashServerId'), 'cashServerId', 'cashServerId')
        changed |= simpleProperty(configInfo.get('cashId'), 'cash.code', 'cashCode')
        changed |= simpleProperty(configInfo.get('shopId'), 'storeID', 'storeID')
        changed |= simpleProperty(configInfo.get('httpHostCS'), 'httpHostCS', 'httpHostCS')
        changed |= simpleProperty(configInfo.get('dictUrlHostCS'), 'dictUrlHostCS', 'dictUrlHostCS')
        changed |= simpleProperty(configInfo.get('dictUrlPortCS'), 'dictUrlPortCS', 'dictUrlPortCS')
        changed |= simpleProperty(configInfo.get('clusterId'), 'clusterId', 'clusterId')
        if not self.grpcCaCertPath:
            changed |= simpleProperty(configInfo.get('grpcHostCS'), 'grpcHostCS', 'grpcHostCS')
        else:
            loggerSync.info(u'Настройка grpcHostCS не изменена со значения %s на %s, т.к. задана настройка grpcCaCertPath, при которой настройка grpcHostCS должна указываться вручную' % (
                    self.grpcHostCS, configInfo.get('grpcHostCS')))
        changed |= simpleProperty(configInfo.get('grpcPortCS'), 'grpcPortCS', 'grpcPortCS')

        protocolVersion = configInfo.get('protocolVersion')
        # в серверном режиме на ubunu 18.04 версия протокола не может быть меньше 5, т.к. на сервере отказались от qpid
        if self.mode == 'server' and (infoReader.platformIsBionic or infoReader.package == 'artixcs-sync') and (
                protocolVersion is not None and int(protocolVersion) < 5 or self.protocolVersion is not None and int(self.protocolVersion) < 5):
            protocolVersion = 5
        changed |= simpleProperty(protocolVersion, 'protocolVersion', 'protocolVersion')

        changed |= simpleProperty(configInfo.get('unloadSalesRequestThroughCS'), 'unloadSalesRequestThroughCS', 'unloadSalesRequestThroughCS')
        changed |= simpleProperty(configInfo.get('isSendDebugDictStatus'), 'isSendDebugDictStatus', 'isSendDebugDictStatus')
        changed |= simpleProperty(configInfo.get('isSendDebugSaleStatus'), 'isSendDebugSaleStatus', 'isSendDebugSaleStatus')

        changedQpidProperties = False
        qpidHostCS = configInfo.get('qpidHostCS')
        httpHostCS = configInfo.get('httpHostCS')
        if qpidHostCS is not None:
            if self.remoteIp != qpidHostCS:
                loggerSync.info(u'Изменяем настройку dept.broker.ip: %s -> %s' % (self.remoteIp, qpidHostCS))
                self.remoteIp = qpidHostCS
                self.remoteBroker = re.sub(r"@(.+):", "@%s:" % qpidHostCS, self.remoteBroker)
                self.__rewriteConfig('dept.broker.ip', qpidHostCS)
                changedQpidProperties = True
                changed = True
        elif httpHostCS is not None and self.remoteIp != httpHostCS:
            loggerSync.info(u'Изменяем настройку dept.broker.ip: %s -> %s' % (self.remoteIp, httpHostCS))
            self.remoteIp = httpHostCS
            self.remoteBroker = re.sub(r"@(.+):", "@%s:" % httpHostCS, self.remoteBroker)
            self.__rewriteConfig('dept.broker.ip', httpHostCS)
            changedQpidProperties = True
            changed = True

        qpidPortCS = configInfo.get('qpidPortCS')
        if qpidPortCS is not None and qpidPortCS != 0 and int(self.remotePort) != int(qpidPortCS):
            loggerSync.info(u'Изменяем настройку dept.broker.port: %s -> %s' % (self.remotePort, qpidPortCS))
            self.remotePort = qpidPortCS
            self.remoteBroker = re.sub(r":\d+", ":%s" % qpidPortCS, self.remoteBroker)
            self.__rewriteConfig('dept.broker.port', qpidPortCS)
            changedQpidProperties = True
            changed = True

        httpPortCS = configInfo.get('httpPortCS')
        if httpPortCS is not None and httpPortCS != 0 and int(self.cashServerRestPort) != int(httpPortCS):
            loggerSync.info(
                u'Изменяем настройку cashServerRestPort: %s -> %s' % (self.cashServerRestPort, httpPortCS))
            self.cashServerRestPort = httpPortCS
            self.httpPortCS = httpPortCS
            self.__rewriteConfig('cashServerRestPort', httpPortCS)
            self.__rewriteConfig('httpPortCS', httpPortCS)
            changed = True

        if changed:
            self.__save()

        return changedQpidProperties

    def __getValue(self, option, defaultValue, required=False):
        """
        Получить значение опции из конфигурационного файла.
        В случае ошибки вернуть значение по умолчанию, если не обязательное значение
        """
        value = None
        try:
            if type(defaultValue) is int:
                value = self.config.getint('Properties', option)
            elif type(defaultValue) is bool:
                value = self.config.getboolean('Properties', option)
            else:
                value = self.config.get('Properties', option)
        except configparser.NoOptionError:
            if required:
                raise
            loggerSync.warn(u'Не найдена настройка %s. Берем значение по умолчанию: %s' % (option, defaultValue))
            value = defaultValue

        loggerSync.info(u'%s - %s' % (option.ljust(40, ' '), str(value).ljust(20, ' ')))
        return value

    def __save(self):
        loggerSync.info(u'Перезаписываем конфигурационный файл')
        with open(PropertyReader.configPath, 'wt') as configfile:
            self.config.write(configfile)

    def __rewriteConfig(self, name, value):
        if not self.config.has_section('Properties'):
            self.config.add_section('Properties')
        self.config.set('Properties', name, value if not isinstance(value, py2to3utils.string_types) else py2to3utils.ensureStr(value))


properties = PropertyReader()
properties.init()
